#include "project.h"
#include "stdio.h"

volatile uint32_t tc_count = 0 ;

#define STR_LEN 128
char str[STR_LEN+1] ;
void print(char *str)
{
    UART_PutString(str) ;
}

CY_ISR(timer_tc_isr)
{
    Timer_ReadStatusRegister() ;
    tc_count++ ;
}

void init_hardware(void)
{
    UART_Start() ;
    
    tc_int_ClearPending() ;
    tc_int_StartEx(timer_tc_isr) ;
    
    Timer_Start() ;
    
    CyGlobalIntEnable; /* Enable global interrupts. */    
}

void splash(void)
{
    snprintf(str, STR_LEN, "Timer Test (%s %s)\n", __DATE__, __TIME__) ;
    print(str) ;
}

void cls(void)
{
    print("\033c") ; /* reset */
    CyDelay(100) ;
    print("\033[2J") ; /* clear screen */
    CyDelay(100) ;
}

void start_measure(void)
{
    Timer_Stop() ;
    Timer_ReadStatusRegister() ;
    Timer_WriteCounter(0) ;
    tc_int_ClearPending() ;
    tc_count = 0 ;
    Timer_Enable() ;
}

uint32_t stop_measure(void)
{
    uint32_t count ;
    Timer_Stop() ;
    count = Timer_ReadPeriod() - Timer_ReadCounter() ;
    if (tc_count) {
        count += tc_count * Timer_ReadPeriod() ;
    }
    return( count ) ;
}

void test_func(uint32_t us)
{
    CyDelayUs(us) ;
}

int main(void)
{
    uint32_t count ;
    uint32_t us  = 0 ;
    
    init_hardware() ;
    
    cls() ;
    
    splash() ;
    
    for(;;)
    {
        start_measure() ;
        test_func(us) ;
        count = stop_measure() ;
        snprintf(str, STR_LEN, "%d: %d us\n", us, count) ;
        print(str) ;
        
        us = us + 10 ;
        if (us > 10000) {
            us = 0 ;
        }
        CyDelay(1000) ;
    }
}

/* [] END OF FILE */
